/*----------------------  BARCODE.C  ---------------------
 *  Prints postal barcodes for a ZIP code per USPS specs
 *  on an HP LaserJet-compatible printer. Usage:
 *
 *                      BARCODE zipcode
 *
 *  where zipcode can use 99999 or 99999-9999 formats
 *
 *  by Andrew Binstock, version 1.3
 *
 *  May be used freely as long as authorship and publica-
 *  tion are acknowledged. (c) Copyright 1990, C Gazette.
 *
 *  Validated for Borland and Microsoft compilers.
 *-------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define LONG     '1'
#define SHORT    '0'
#define ESC      0x1B  /* the escape character */
#define FORMFEED 0x0C  /* form-feed            */

FILE *prt;

void print_bar ( char );
void print_bar_code ( char * );
void print_digit_barcode ( char * );

void print_bar_code ( char *zip )
{
    static char * pattern [10] =
    {
                /* 74210 */  /* The value of each position */
        /* 0 */   "11000",
        /* 1 */   "00011",
        /* 2 */   "00101",
        /* 3 */   "00110",
        /* 4 */   "01001",
        /* 5 */   "01010",
        /* 6 */   "01100",
        /* 7 */   "10001",
        /* 8 */   "10010",
        /* 9 */   "10100"
    };

    char *p;
    int i, j, len, sum;
    i = sum = 0;

    len = strlen ( zip );
    if ( len != 5 && len != 10 )
    {
        fprintf ( stderr, "zip %s must be 5 or 10 characters", zip );
        exit ( EXIT_FAILURE );
    }

    while ( *zip )
    {
        if ( *zip == '-' )          /* skip the dash in ZIP+4 */
        {
            zip += 1;
            continue;
        }

        /* since zip is going to be converted into an index
           to pattern, we must make sure it conatinas the
           data we expect, namely digits.   */

        if ( !( isdigit ( *zip )))
        {
           fputs ( "Zip Code Error", prt );
           exit ( EXIT_FAILURE );
        }

        j = *zip - '0';             /* convert to a number */

        if ( i == 0 )               /* no digits done yet? */
            print_bar ( LONG );     /* then do initial bar */

        p = pattern [ j ];          /* point p at the pattern */

        print_digit_barcode ( p );  /* print it */

        i   += 1;                   /* count digits processed */
        sum += j;                   /* sum of zip code digits */
        zip += 1;                   /* go on to next digit    */
    }

    sum %= 10;                      /* remainder of sum/10 */

    p = pattern [ 10 - sum ];       /* check digit = 10 - sum */
    print_digit_barcode ( p );

    print_bar ( LONG );             /* closing bar */
}

void print_digit_barcode ( char * p )
{
    while ( *p )
    {
        print_bar ( *p );
        p += 1;
    }
}

void print_bar ( char size )
{
    if ( size == SHORT )
        fprintf ( prt, "%c*p+21Y", ESC );

    /* bar width is .015 - .025 inch  */
    /* 6 dots = .020 inch             */

    fprintf ( prt, "%c*c%dA", ESC, 6 );

    /* bar height is:
        LONG:   .115 - .135 inch  or  35 - 41 dots
        SHORT:  .040 - .060 inch  or  14 - 20 dots

        we use: LONG 38 dots, SHORT 17 dots
     */

    fprintf ( prt, "%c*c%dB", ESC,
               ( size == SHORT ? 17 : 38 ));

    /* specify print as black */

    fprintf ( prt, "%c*c0P", ESC );

    /* horizontal spacing from start of one bar to start
       of next bar is .045 - .050 inch, or 14 - 15 dots.
       We'll use a relative move of 15 dots.
     */

    fprintf ( prt, "%c*p+15X", ESC );

    if ( size == SHORT )
        fprintf ( prt, "%c*p-21Y", ESC );

}

main ( int argc, char * argv[] )
{
    if ( argc < 2 )
    {
        fputs ( "Usage: BARCODE zipcode", stderr );
        exit ( EXIT_FAILURE );
    }

    prt = fopen ( "PRN", "wb" );
    if ( prt == NULL )
    {
        fputs ("Error opening printer\n", stderr );
        exit ( EXIT_FAILURE );
    }

    fprintf ( prt, "\n\r\tBarcode for %s is:  ", argv[1] );
    print_bar_code ( argv[1] );
    fprintf ( prt, "%c", FORMFEED );   /* eject the page */
    return ( EXIT_SUCCESS );
}